/** 
	@namespace This is the first namespace.
*/
ns1 = {};

/** 
	This is the second namespace.
	@namespace
*/
ns1.ns2 = {};

/**
	This part of ns1.ns2
	@constructor
*/
ns1.ns2.Function1 = function() {
};

ns1.staticFunction = function() {
};

/** A static field in a namespace. */
ns1.ns2.staticField = 1;
