<header role="banner">
	<h1>
		<a href="http://www.webitup.fr"><img src="img/logo.png" alt="Web It Up" /></a>
	</h1>
	<nav role="navigation">
		<ul>
			<li><a href="./">Documentation home</a></li>
			<li><a href="./">Class index</a></li>
			<li><a href="files.html">Files index</a></li>
		</ul>
	</nav>
</header>