
/**
 * Builtin object.
 * @class
 * @name Array
 */
 
/**#@+
 * Extension to builtin array.
 * @memberOf Array
 * @method
 */
 
/**
 * @returns Boolen if some array members...
 */
Array.prototype.some = function(){};

/**
 * Change every element of an array.
 * @returns Filtered array copy.
 */
Array.prototype.filter = function(){};

/**#@-*/


/**
 * A first in, first out data structure.
 * @constructor
 */
Queue = function(){};

/**#@+
 * Extension to Queue.
 * @memberOf Queue
 */

rewind = function(){
}

// should close automatically here.