/**
 * @overview This "library" contains a 
 *               lot of classes and functions.
 * @example
 <pre>
	var x (x < 1);
	alert("This 'is' \"code\"");
 </pre>
 * @name My Cool Library
 * @author 	Joe Smith jsmith@company.com
 * @version 	0.1 
 */
 
/** 
 * Gets the current foo 
 * @param {String} fooId	The unique identifier for the foo.
 * @return {Object}	Returns the current foo.
 */
function getFoo(fooID){
}