/*
 * TABLE OF CONTENTS:
 * - Browser reset
 * - HTML elements
 * - JsDoc styling
 * - Media query check
 */






/*
 * HTML ELEMENTS
 */

body {
	padding: 1% 4% 1% 4%;
}

/*
 * HTML ELEMENTS
 */



 
 
/*
 * BEGIN JSDOC
 */

/* Start menu */
div.index div.menu {
	position: fixed;
	top: 0;
	right: 0;
	-moz-border-radius-bottomleft: 15px;
	-webkit-border-bottom-left-radius: 15px;
	-border-bottom-left-radius: 15px;
	padding: 4px 5px 8px 10px;
	-moz-box-shadow: 0px 0px 10px #c4c4c4;
	-webkit-box-shadow: 0px 0px 10px #c4c4c4;
	box-shadow: 0px 0px 10px #c4c4c4;
	background-color: rgba(255, 255, 255, 0.9);
}

div.index input.classFilter {
	display: none;
}

div.index div.indexLinks a {
	float: right;
	clear: both;
	font-size: 1.1em;
}

div.index *.heading1 {
	display:none;
}

div.index ul.classList {
	display:none;
}

div.index div.fineprint {
	display:none;
}

div.indexStatic {
	display: none;
}
/* End menu */



/* Start content */
div.content *.classTitle {
	margin-right: 60px;
	margin-bottom: 15px;
}

div.content div.intro {
	margin: 15px 0 35px;
}

div.content p.description.summary {
	margin-bottom: 0.2em;
}

div.content div.props {
	margin: 1.5em -2% 0 -2%;
	padding: 2%;
}

table.summaryTable {
	position: relative;
	left: -10px;
	width: 100%;
	border-collapse: collapse;
	box-sizing: content-box;
	-moz-box-sizing: content-box;
	-webkit-box-sizing: content-box;
	-ms-box-sizing: content-box;
	-o-box-sizing: content-box;
	-icab-box-sizing: content-box;
	-khtml-box-sizing: content-box; 
}

*.sectionTitle {
	padding: 0 10px 10px 0;
}
caption.sectionTitle {
	padding-left: 10px;
}

table.summaryTable td,
table.summaryTable th {
	padding: 0px 10px 10px 10px;
}
table.summaryTable tr:last-child td {
	padding-bottom: 0;
}

table.summaryTable td.attributes {
	width: 35%;
}

table.summaryTable td.nameDescription {
	width: 65%
}



dl.detailList {
	margin-top: 0.5em;
}

dl.detailList.nomargin + dl.detailList.nomargin {
	margin-top: 0;
}

dl.detailList dt {
	display: inline;
	margin-right: 5px;
}

dl.detailList dt:before {
	display: block;
	content: "";
}

dl.detailList dd {
	display: inline;
}

dl.detailList.params dt {
	display: block;
}
dl.detailList.params dd {
	display: block;
	padding-left: 2em;
	padding-bottom: 0.4em;
}




ul.fileList li {
	margin-bottom: 1.5em;
}



.fixedFont.heading {
	margin-bottom: 0.5em;
}

pre.code {
	margin: 10px 0 10px 0;
	padding: 10px;
	border: 1px solid #ccc;
	-moz-border-radius: 2px;
	-webkit-border-radius: 2px;
	border-radius: 2px;
}
/* End content */

/*
 * END JSDOC
 */







/*
 * START MEDIA QUERY CHECK
 */

.cssLoadCheck {
	position: absolute;
	top: -99999px;
	left: -99999px;
	border: 0;
	width: 100px;
	padding: 0;
	overflow: hidden;
}

/*
 * END MEDIA QUERY CHECK
 */

