/**
 * @constructor
 * @param person The person.
 * @param {string} person.name The person's name.
 * @config {integer} age The person's age.
 * @config [id=1] Optional id number to use.
 * @param connection
 */
function Contact(person, connection) {

}

/**
 * @constructor
 * @param persons
 * @config {string} Father The paternal person.
 * @config {string} Mother The maternal person.
 * @config {string[]} Children And the rest.
 */
function Family(/**Object*/persons) {

}
