/*
 * TABLE OF CONTENTS:
 * - JsDoc styling
 * - Media query check
 */






/*
 * BEGIN JSDOC
 */

/* Start menu */
div.index {
	position: fixed;
	top: 0;
	bottom: 0;
	float: left;
	width: 30%;
	min-width: 100px;
	max-width: 300px;
	padding: 0 0 10px 0;
	overflow: auto;
}

div.index *.heading1 {
	padding: 8px 0 0 0;
}

div.index div.menu {
	margin: 0 15px 0 -15px;
	-moz-border-radius-bottomright: 15px;
	-webkit-border-bottom-right-radius: 15px;
	-border-bottom-right-radius: 15px;
  	padding: 15px 15px 15px 30px;
	-moz-box-shadow: 0px 0px 10px #c4c4c4;
	-webkit-box-shadow: 0px 0px 10px #c4c4c4;
	box-shadow: 0px 0px 10px #c4c4c4;
	background-color: rgba(255, 255, 255, 0.5);
}

div.index div.indexLinks {
	margin-top: 13px;
	position: absolute;
	right: 30px;
}

div.index div.indexLinks a {
	color: #999999;
	text-transform: lowercase;
}

div.index div.indexLinks a:first-child {
	margin-right: 3px;
	border-right: 1px solid #999999;
	padding-right: 5px;
}

div.index input.classFilter {
	margin-bottom: 4px;
	width: 100%;
	border-width: 1px;
	border-style: solid;
	border-color: #CCCCCC #999999 #999999 #CCCCCC;
	-moz-border-radius: 3px;
	-webkit-border-radius: 3px;
	-border-radius: 3px;
}

div.index ul.classList a {
	line-height: 1.3em;
}

div.index ul.classList a + a {
	margin-left: 0.5em;
}

div.index div.fineprint {
	margin: 1em 0 0 15px;
	color: #777;
	font-size: 0.9em;
}

div.index div.fineprint a {
	color: #777;
}

div.indexStatic {
	position: static;
	min-height: 1em;
}
/* End menu */


/* Start content */
div.content {
	float: left;
	width: 70%;
	min-width: 300px;
	max-width: 600px;
}
div.innerContent {
	padding: 0 0 0 2.5em;
}

div.content ul,
div.content ol {
	margin-bottom: 3em;
}

div.content ul.methodDetail {
	margin-bottom: 0;
}

div.content *.classTitle {
	position: relative;
	left: -10px;
	margin: -30px 0 15px 0;
	-moz-border-radius: 15px;
	-webkit-border-radius: 15px;
	border-radius: 15px;
  	padding: 25px 15px 15px 15px;
	background-color: #FFFFFF;
	background-color: rgba(255, 255, 255, 0.5);
	-moz-box-shadow: 0px 0px 10px #c4c4c4;
	-webkit-box-shadow: 0px 0px 10px #c4c4c4;
	box-shadow: 0px 0px 10px #c4c4c4;
}

div.content div.intro {
	margin: 15px 0 45px
}

div.content p.summary {
	margin-bottom: 0.5em;
}

div.content ul.summary {
	margin-bottom: 1.5em;
}

div.content ul *.classname a,
div.content ul *.filename a {
	font-family: Consolas, "Courier New", Courier, monospace;
	text-decoration: none;
	font-weight: bold;
}
div.content ul *.classname a:hover,
div.content ul *.filename a:hover {
	text-decoration: underline;
}

div.content div.props {
	position: relative;
	left: -10px;
	margin-bottom: 2.5em;
	padding: 10px 15px 15px 15px;
	overflow: hidden;
}

div.content div.hr {
	margin: 0 10px 0 0;
	height: 4em;
}



table.summaryTable {
	position: relative;
	left: -10px;
	width: 100%;
	border-collapse: collapse;
	box-sizing: content-box;
	-moz-box-sizing: content-box;
	-webkit-box-sizing: content-box;
	-ms-box-sizing: content-box;
	-o-box-sizing: content-box;
	-icab-box-sizing: content-box;
	-khtml-box-sizing: content-box; 
}

*.sectionTitle {
	padding: 0 10px 10px 0;
}
caption.sectionTitle {
	padding-left: 10px;
}

table.summaryTable td,
table.summaryTable th {
	padding: 0px 10px 10px 10px;
}
table.summaryTable tr:last-child td {
	padding-bottom: 0;
}

table.summaryTable td.attributes {
	width: 35%;
}

table.summaryTable td.nameDescription {
	width: 65%
}



dl.detailList {
	margin-top: 0.5em;
}

dl.detailList.nomargin + dl.detailList.nomargin {
	margin-top: 0;
}

dl.detailList dt {
	display: inline;
	margin-right: 5px;
}

dl.detailList dt:before {
	display: block;
	content: "";
}

dl.detailList dd {
	display: inline;
}

dl.detailList.params dt {
	display: block;
}
dl.detailList.params dd {
	display: block;
	padding-left: 2em;
	padding-bottom: 0.4em;
}




ul.fileList li {
	margin-bottom: 1.5em;
}



.fixedFont.heading {
	margin-bottom: 0.5em;
}

pre.code {
	margin: 10px 0 10px 0;
	padding: 10px;
	border: 1px solid #ccc;
	-moz-border-radius: 2px;
	-webkit-border-radius: 2px;
	border-radius: 2px;
}
/* End content */

.clear {
	clear: both;
	width: 100%;
	min-height: 0;
}

/*
 * END JSDOC
 */







/*
 * START MEDIA QUERY CHECK
 */

.cssLoadCheck {
	position: absolute;
	top: -99999px;
	left: -99999px;
	border: 0;
	width: 100px;
	padding: 0;
	overflow: hidden;
}

/*
 * END MEDIA QUERY CHECK
 */

